/* isBlankTest.sas
   purpose: to automatize the test of various implementation of isBlank
            macro ("contenders") with various input parameter values.
            this is an appendix of the conference paper titled "Is
            this macro parameter blank?"
   input  : nothing
   output : given v parameter values and c contenders, it will
            write out v * c sas source code files (.sas) and
            a log files (.log) for each, in the isBlankTest sub-
            directory, under the WORK directory (which will be cleaned
            if already exists)
   note   : ran with SAS 9.1.3 SP4 for Windows. no guarantees what so
            ever, comments are welcome.
   ver    : 0.4 (2008-10-09)
   by     : chang y chung and john king
   contact: chang_y_chung@hotmail.com
*/

options noovp nocenter;

*cd to work directory ------------------------------------------------;;
x cd %sysfunc(pathname(WORK));

*files and directories -----------------------------------------------;;
filename test "isBlankTest"; *test programs will be written here;
filename exec %sysfunc(quote(%sysfunc(sysget(sasroot))\sas.exe));
filename incl temp; *for %include;

*input parameter values ----------------------------------------------;;
data vs;
  infile cards truncover firstobs=2;
  input vid $ blank $ v $ 6-40;
  /* (nothing) will be converted to nothing later */
cards;
----+----1----+----2----+----3----+----4
01 1 (nothing)
02 1 %str( )
03 0 &
04 0 A>B+C>D
05 0 %bquote(Tom&Jerry)
06 0 %nrstr(%()
07 0 1 or
08 0 %sysfunc(repeat(X,65533-1))  
09 1 %qsysfunc(repeat(%str( ),10000-1))
10 0 43876106244387610624
;
run;

*contenders ----------------------------------------------------------;;
data cs;
  infile cards truncover firstobs=2;
  input cid $ c $ 3-40;
cards;
----+----1----+----2----+----3----+----4
01 &param eq
02 %bquote(&param)=
03 %nrbquote(&param)=
04 %superq(param)=
05 "&param" = ""
06 %length(&param) = 0
07 %length(%qleft(%qtrim(&param))) = 0
08 %sysevalf(%superq(param)=,boolean)
;
run;

*cartesian product. also add a var, vc -------------------------------;;
data vcs;  
  set vs;
  do p = 1 to n;
    set cs point=p nobs=n;
    vc = catt("v", vid, "c", cid);
    output;
  end;
run;

*test macro code template --------------------------------------------;;
data tmpl;
  infile cards truncover firstobs=2;
  input line $char55.;
cards4;
----+----1----+----2----+----3----+----4----+----5----+
/* test macro vid=[vid] cid=[vid]*/
options ls=max ps=max;
%macro test(param);
  %put &prefix IN entry=Success;
  %if [c] %then %let res=1;
  %else %let res=0;
%mend  test;

/* prepare for the invocation */
%let prefix = NOTE: (%str(TEST));
%put &prefix WHAT vid=[vid] cid=[cid];
%let start = %sysfunc(time()); 
%let res=;

/* macro call */
%test([v])

/* output results */
%put &prefix RES
  res=%sysfunc(ifc(&res=[blank],Correct,Incorrect))
  time=%sysevalf(%sysfunc(time())-&start);
;;;;
run;

*create the test directory or clean up the existing one --------------;;
data _null_;
  length testpath filepath dn $256 fn $8;
  testpath = pathname("test");
  if not fileexist(testpath) then do;
    dn = dcreate(scan(testpath, -1, "\/"));
    if missing(dn) then error "ERROR: cannot create dir " testpath;
  end;
  did = dopen("test");
  if did = 0 then error "ERROR: cannot open dir " testpath;
  dnum = dnum(did);
  if not missing(dnum) then do i = 1 to dnum;
    filepath = catx("/", testpath, dread(did, i));
    rc = filename(fn, filepath);
    rc = fdelete(fn);
    if rc^ = 0 then error "ERROR: cannot delete file " filepath;
    rc = filename(fn,"");
  end;
  rc = dclose(did);
run;

*write out test sas files and test driver to be included -------------;;
data _null_;

  set vcs;

  length testpath fname line $256;
  testpath = pathname("test");
  fname = catx("\", testpath, catx(".", vc, "sas"));

  file dummy filevar=fname; *test sas files;
  do p = 1 to n;
    set tmpl point=p nobs=n;
    line = tranwrd(line, "[vid]", trimn(vid));
    line = tranwrd(line, "[cid]", trimn(cid));
    line = tranwrd(line, "[v]", trimn(ifc(v="(nothing)","",v)));
    line = tranwrd(line, "[c]", trimn(c));
    line = tranwrd(line, "[blank]", trimn(blank));
    len = length(line);
    put line $varying. len; 
  end;

  file incl; *test driver to be included;
  length exec opts sysin log cmd $256;
  exec = quote(trimn(pathname("exec")));
  opts = "-nocenter -noovp -nosplash -rsasuser";
  sysin = catx(" ", "-sysin", quote(trimn(fname)));
  log = catx(" ", "-log", quote(trimn(testpath)));
  cmd = quote(catx(" ", exec, opts, sysin, log));
  len = length(cmd);
  status = vc;
  put "systask command" +1 cmd $varying. len +1 status= +1 "wait;";
  put '%put NOTE: ' status '=&' status ';';
run;  

*run all test sas files ----------------------------------------------;;
%include incl/ source2;

*read the log files into an analysis dataset -------------------------;;
data results;
  length vid cid $2 entry $7 res $9;
  retain vid cid entry res " " warned erred time .;
  infile test('*.log') eov=eov eof=eof truncover;
  input @1 msgType $ @1 @("NOTE: (TEST)") noteType $ @;
  if eov then goto eof;
  select(msgType);
    when("ERROR:") if entry="Success" & missing(res) then erred = 1;
    when("WARNING:") if entry="Success" /* & warned(res) */ then warned = 1;
    when("NOTE:") do;
      select(noteType);
        when("WHAT") input vid= cid=;
        when("IN") input entry=;
        when("RES") input res= time=;
        otherwise;
      end;
    end;
    otherwise;
  end;
  input;
  return;
eof:
  keep vid--time;
  output;
  eov = 0;
  call missing(entry, res, warned, erred, time);
run;

*report test results -------------------------------------------------;;
proc print data=results noobs;
run;

/* output: on lst
vid    cid     entry        res       warned    erred      time

01     01     Success    Correct         .        .      0.00100
01     02     Success    Correct         .        .      0.00100
01     03     Success    Correct         .        .      0.00100
01     04     Success    Correct         .        .      0.00100
01     05     Success    Correct         .        .      0.00100
01     06     Success    Correct         .        .      0.00100
01     07     Success    Correct         .        .      0.07100
01     08     Success    Correct         .        .      0.00100
02     01     Success    Correct         .        .      0.00100
02     02     Success    Correct         .        .      0.00100
02     03     Success    Correct         .        .      0.00100
02     04     Success    Correct         .        .      0.00100
02     05     Success    Incorrect       .        .      0.00100
02     06     Success    Incorrect       .        .      0.00100
02     07     Success    Correct         .        .      0.05300
02     08     Success    Correct         .        .      0.00100
03     01     Success    Incorrect       .        1      0.00100
03     02     Success    Incorrect       .        1      0.00100
03     03     Success    Correct         .        .      0.00100
03     04     Success    Correct         .        .      0.00100
03     05     Success    Correct         .        .      0.00100
03     06     Success    Correct         .        .      0.00100
03     07     Success    Correct         .        .      0.01000
03     08     Success    Correct         .        .      0.00000
04     01     Success    Incorrect       .        1      0.00100
04     02     Success    Correct         .        .      0.00100
04     03     Success    Correct         .        .      0.00000
04     04     Success    Correct         .        .      0.00000
04     05     Success    Correct         .        .      0.00100
04     06     Success    Correct         .        .      0.00100
04     07     Success    Correct         .        .      0.06100
04     08     Success    Correct         .        .      0.00100
05     01     Success    Incorrect       .        1      0.00100
05     02     Success    Incorrect       1        1      0.00100
05     03     Success    Correct         .        .      0.00100
05     04     Success    Correct         .        .      0.00100
05     05     Success    Correct         .        .      0.00100
05     06     Success    Correct         .        .      0.00200
05     07     Success    Correct         1        .      0.00800
05     08     Success    Correct         .        .      0.00000
06     01     Success    Correct         .        .      0.00100
06     02     Success    Correct         .        .      0.00100
06     03     Success    Correct         .        .      0.00100
06     04     Success    Correct         .        .      0.00000
06     05     Success    Correct         .        .      0.00000
06     06     Success    Correct         .        .      0.00100
06     07     Success    Correct         .        .      0.00900
06     08     Success    Correct         .        .      0.00100
07     01     Success    Incorrect       .        .      0.00000
07     02     Success    Correct         .        .      0.00100
07     03     Success    Correct         .        .      0.00100
07     04     Success    Correct         .        .      0.00100
07     05     Success    Correct         .        .      0.00100
07     06     Success    Correct         .        .      0.00200
07     07     Success    Correct         .        .      0.00900
07     08     Success    Correct         .        .      0.00100
08     01     Success    Correct         .        .      0.04700
08     02     Success    Correct         .        .      0.04500
08     03     Success    Correct         .        .      0.07000
08     04     Success    Correct         .        .      0.06500
08     05     Success    Correct         .        .      0.03100
08     06     Success    Correct         .        .      0.06300
08     07     Success    Incorrect       .        1      0.11100
08     08     Success    Correct         .        .      0.03900
09     01     Success    Correct         .        .      0.03300
09     02     Success    Correct         .        .      0.03500
09     03     Success    Correct         .        .      0.03500
09     04     Success    Correct         .        .      0.04200
09     05     Success    Incorrect       .        .      0.03400
09     06     Success    Incorrect       .        .      0.04200
09     07     Success    Correct         .        .      3.63400
09     08     Success    Correct         .        .      0.04200
10     01     Success    Incorrect       .        1      0.00100
10     02     Success    Incorrect       .        1      0.00100
10     03     Success    Incorrect       .        1      0.00100
10     04     Success    Incorrect       .        1      0.00100
10     05     Success    Correct         .        .      0.00100
10     06     Success    Correct         .        .      0.00200
10     07     Success    Correct         .        .      0.04500
10     08     Success    Correct         .        .      0.00100
*/

